<?php

namespace FiloBlu\Storelocator\Helper;

class Data extends \Magento\Framework\App\Helper\AbstractHelper
{
    const CONFIG_IMAGE_DIR_PREFIX = 'filoblu/storelocator/';
    const STORES_IMAGE_DIR_PREFIX = 'filoblu/storelocator/stores/image/';

    protected $_storeManager;
    protected $_returntostore;
    /**
     * @var \FiloBlu\Storelocator\Model\TagsFactory
     */
    private $_tagsFactory;

    public function __construct(\Magento\Framework\App\Helper\Context $context,
                                \Magento\Store\Model\StoreManagerInterface $storeManager,
                                \Magento\Framework\View\Asset\Repository $assetRepo,
                                \FiloBlu\Storelocator\Model\Rma\Returntostore $_returntostore,
                                \FiloBlu\Storelocator\Model\TagsFactory $_tagsFactory
    ) {
        $this->_storeManager = $storeManager;
        $this->scopeConfig = $context->getScopeConfig();
        $this->assetRepo = $assetRepo;
        $this->request = $context->getRequest();
        $this->urlBuilder = $context->getUrlBuilder();
        $this->_returntostore = $_returntostore;
        $this->_tagsFactory = $_tagsFactory;
    }

    public function getConfig($config_path)
    {
        return $this->scopeConfig->getValue(
            $config_path,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    public function geocode($address)
    {
        // url encode the address
        $address = urlencode($address);

        // google map geocode api url
        $key = $this->getConfig('filoblu_storelocator_section/general/gmaps_api_key_server_side');
        $url = $this->getConfig('filoblu_storelocator_section/geocode/gmaps_geocode_api_url') . "?address={$address}&key={$key}";

        // get the json response
        $resp_json = file_get_contents($url);

        // decode the json
        $resp = json_decode($resp_json, true);

        // response status will be 'OK', if able to geocode given address
        if ($resp['status'] == 'OK') {

            // get the important data
            $lati = $resp['results'][0]['geometry']['location']['lat'];
            $longi = $resp['results'][0]['geometry']['location']['lng'];
            $formatted_address = $resp['results'][0]['formatted_address'];

            // verify if data is complete
            if ($lati && $longi && $formatted_address) {

                // put the data in the array
                $data_arr = array();

                array_push(
                    $data_arr,
                    $lati,
                    $longi,
                    $formatted_address
                );

                return $data_arr;

            } else {
                return false;
            }

        } else {
            return false;
        }
    }

    public function getBaseUrlConfigImage()
    {
        return $this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA) . SELF::CONFIG_IMAGE_DIR_PREFIX;
    }

    public function getBaseUrlStoresImage()
    {
        return $this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA) . SELF::STORES_IMAGE_DIR_PREFIX;
    }

    public function getBaseUrl()
    {
        return $this->_storeManager->getStore()->getBaseUrl();
    }

    public function getViewFileUrl($fileId, array $params = [])
    {
        try {
            $params = array_merge(['_secure' => $this->request->isSecure()], $params);
            return $this->assetRepo->getUrlWithParams($fileId, $params);
        } catch (\Magento\Framework\Exception\LocalizedException $e) {
            return $this->urlBuilder->getUrl('', ['_direct' => 'core/index/notFound']);
        }
    }
    public function getTagFiltersEnabled(){
        return  $this->scopeConfig->getValue('filoblu_storelocator_section/return_to_store/enable_tag_filter_by_category', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }

    public function getTagCategoryFilterIds() {
        $params = false;
        $enabled =  $this->scopeConfig->getValue('filoblu_storelocator_section/return_to_store/enable_tag_filter_by_category', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $return = false;
        if($enabled == '1'){
            $params = $this->scopeConfig->getValue('filoblu_storelocator_section/return_to_store/tag_filter_by_category', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
            $params = unserialize($params);
            foreach($params as $ids){
                if(array_key_exists('tag_category_filter_id' , $ids)){
                    $return = $ids['tag_category_filter_id'];
                    $return = explode(',', $return);
                }
            }
        }
        return $return;
    }

    public function getTagFiltersLabels() {
        $params = false;
        $enabled =  $this->getTagFiltersEnabled();
        $return = false;
        if($enabled == '1'){
            $params = $this->scopeConfig->getValue('filoblu_storelocator_section/return_to_store/tag_filter_by_category', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
            $params = unserialize($params);
            foreach($params as $label){
                if(array_key_exists('tag_filter_label' , $label)){
                    $return = $label['tag_filter_label'];
                    $return = explode(',', $return);
                }
            }
        }
        return $return;
    }



    public function getTagFilterHandle($orderId = null){
        // READ TAGS
        $alltags = $this->_tagsFactory->create()->getAllAvailableTags();
        $tags = array();

        if(count($alltags) > 0){
            foreach($alltags as $tag){
                $tags[$tag->getName()] = $tag->getId();
            }
        }

        if(!is_null($orderId) &&
            $this->getTagFiltersEnabled() == 1 &&
            is_array($this->getTagFiltersLabels()) &&
            is_array($this->getTagCategoryFilterIds())){

            $categories = $this->_returntostore->getItemsCategoriesByOrderId($orderId);
            $countCat = count($categories);
            $setting = array();
            $flatted_categories = array();
            $counter = 0;
            array_walk_recursive($categories, function($a) use (&$flatted_categories) { $flatted_categories[] = $a; }); // flat multidimensional array
            $categories = $flatted_categories;
            $tag_setting = $this->getTagFiltersLabels();
            if(is_array($this->getTagCategoryFilterIds())){
                $setting = $this->getTagCategoryFilterIds();
            }
            foreach($categories as $cat){
                if(in_array($cat, $setting)){ // $cat = all my categories  $setting = my outlet ids
                    $counter++;
                }
            }


            if($counter > 0){
                // outlet result -> consider Storelocator Setting tag ids

                foreach($tags as  $key => $tag_ids) {
                    if(is_array($tag_setting) && !in_array($key, $tag_setting)){
                        unset($tags[$key]);
                    }
                }
            }else {
                // monobrand result -> consider tag ids excluded
                foreach($tags as  $key => $tag_ids) {
                    if(is_array($tag_setting) && in_array($key, $tag_setting)){
                        unset($tags[$key]);
                    }
                }
            }

        }
        return $tags;
    }

    public function getItemsCategoriesByOrderId($orderId = null){
        return $this->_returntostore->getItemsCategoriesByOrderId($orderId);
    }
//\Magento\Framework\App\ObjectManager::getInstance()
//->get('Psr\Log\LoggerInterface')->debug(print_r($model->getData(),true));
}
